@@ -14,9 +14,13 @@ import numpy as np
 
 from datetime import  timedelta
 
+from binance.client import Client
+
 class NaPoleonBinanceFutureBot(object):
 
     def __init__(self, BINANCE_PUBLIC, BINANCE_PRIVATE):
+
+        client = Client(BINANCE_PUBLIC, BINANCE_PRIVATE)
         self.apiKey = BINANCE_PUBLIC
         self.secret = BINANCE_PRIVATE
         Binancecle2 = {
@@ -51,6 +55,50 @@ class NaPoleonBinanceFutureBot(object):
         self.cancel_all_order('TRX/USDT')
 
 
+    def get_spot_universe(self):
+
+        BASE_URL = 'https://api.binance.com'
+        PATH = '/sapi/v1/accountSnapshot'
+
+        headers = {
+            'X-MBX-APIKEY': self.apiKey
+        }
+        timestamp = int(time.time() * 1000)
+
+        params = {
+            'type': 'FUTURES',
+            'recvWindow': 60000,
+            'timestamp': timestamp,
+            'startTime': startTime,
+            'endTime': endTime,
+            'limit':300
+
+        }
+        query_string = urllib.parse.urlencode(params)
+        params['signature'] = hmac.new(self.secret.encode('utf-8'), query_string.encode('utf-8'),
+                                       hashlib.sha256).hexdigest()
+        url = urljoin(BASE_URL, PATH)
+        print(url)
+        r = requests.get(url, headers=headers, params=params)
+        dataSet = r.json()
+
+        snapshots = dataSet['snapshotVos']
+
+        concatenated_df = None
+        for me_snapshot in snapshots:
+            data_df = pd.DataFrame(me_snapshot['data']['assets'])
+            data_df['updateTime'] = me_snapshot['updateTime']
+            if concatenated_df is None:
+                concatenated_df = data_df.copy()
+            else :
+                concatenated_df = pd.concat([concatenated_df, data_df])
+
+        concatenated_df['date'] = pd.to_datetime(concatenated_df['updateTime'], unit='ms')
+        concatenated_df['date'] = concatenated_df['date'] + timedelta(seconds=1)
+
+        return concatenated_df
+
+
     def get_futures_balance_history(self, start_time, end_time):
         startTime = int(start_time.timestamp() * 1000)
         endTime = int(end_time.timestamp() * 1000)
