@@ -202,6 +202,24 @@ def flat_numba_drift_rebal(previous_weights, current_prices, previous_prices):
     new_weights = previous_weights*current_prices/previous_prices
     return new_weights
 
+
+@njit()
+def numba_indice_rebalancing_ok(daily_rebal, rebal_data, price_data, previous_price_data, target_weights):
+    values = np.zeros(price_data.shape[0])
+    weights = np.zeros((price_data.shape[0],price_data.shape[1]))
+    for i in range(price_data.shape[0]):
+        if i == 0:
+            weights[i, :] = target_weights
+            values[i] = np.sum(target_weights)
+            continue
+        must_rebalance = rebal_data[i]
+        if daily_rebal:
+            weights[i, :] = values[i-1]*flat_numba_drift_rebal(target_weights, price_data[i,:], previous_price_data[i,:])
+        else :
+            weights[i, :] = flat_numba_drift_rebal(weights[i - 1, :], price_data[i,:], previous_price_data[i,:])
+        values[i] = np.sum(weights[i, :])
+    return weights, values
+
 @njit()
 def numba_indice_rebalancing(early_rebal, rebal_data, price_data, previous_price_data, target_weights):
     values = np.zeros(price_data.shape[0])
@@ -215,7 +233,6 @@ def numba_indice_rebalancing(early_rebal, rebal_data, price_data, previous_price
         if early_rebal and i >= 1:
             previous_weights_have_drifted = np.sum(weights[i - 1, :] > target_weights) > 0
             must_rebalance = must_rebalance or previous_weights_have_drifted
-
         if must_rebalance:
             weights[i, :] = values[i-1]*flat_numba_drift_rebal(target_weights, price_data[i,:], previous_price_data[i,:])
         else :
@@ -256,6 +273,105 @@ def numba_indice_rebalancing_bestof(early_rebal, rebal_data, price_data, previou
         values[i] = np.nansum(weights[i, :])
     return weights, values
 
+def numba_indice_rebalancing_bestof_ok(daily_rebal, rebal_data, price_data, previous_price_data, returns_data, bestofcount, delay):
+    target_weights =  np.zeros(price_data.shape[1])
+    values = np.zeros(price_data.shape[0])
+    money_on_stock = np.zeros((price_data.shape[0],price_data.shape[1]))
+
+    past_returns = np.nanmean(returns_data[0:bestofcount], 0)
+    chosen_constituents = np.argsort(-past_returns)[:bestofcount]
+    target_weights[chosen_constituents] = 1 / bestofcount
+
+    for i in range(delay, price_data.shape[0]):
+        if i == delay:
+            last_rebal = i
+            money_on_stock[i, :] = target_weights
+            values[i] = np.sum(target_weights)
+            continue
+        must_rebalance = rebal_data[i]
+        if must_rebalance:
+            past_returns = np.nanmean(returns_data[last_rebal :i],0)
+            chosen_constituents = np.argsort(-past_returns)[:bestofcount]
+            target_weights = np.zeros(price_data.shape[1])
+            target_weights[chosen_constituents] = 1/bestofcount
+            money_on_stock[i, :] = values[i-1]*flat_numba_drift_rebal(target_weights, price_data[i,:], previous_price_data[i,:])
+            last_rebal = i
+        elif daily_rebal :
+            money_on_stock[i, :] = values[i-1]*flat_numba_drift_rebal(target_weights, price_data[i,:], previous_price_data[i,:])
+        else:
+            money_on_stock[i, :] = flat_numba_drift_rebal(money_on_stock[i - 1, :], price_data[i,:], previous_price_data[i,:])
+        values[i] = np.nansum(money_on_stock[i, :])
+    return money_on_stock, values
+
+def pre_numba_indice_rebalancing_bestof_ok(price_data= None, inception_date=None, initial_value= 100., reshuffling_method = None , daily_rebalancing = False, last = True, drop_na = True, bestofcount = 5):
+
+    constituents = list(price_data.columns)
+    constituents.sort()
+
+    assert len([constituent for constituent in constituents if constituent in price_data.columns]) == len(constituents)
+
+
+    previous_prices_underlying = []
+    for me_constituent in constituents:
+        previous_prices_constituent =  f'previous_price_{me_constituent}'
+        previous_prices_underlying.append(previous_prices_constituent)
+        price_data[previous_prices_constituent] = price_data[me_constituent].shift(1)
+
+    returns_underlying = []
+    for me_constituent in constituents:
+        returns_constituent =  f'return_{me_constituent}'
+        returns_underlying.append(returns_constituent)
+        price_data[returns_constituent] = price_data[me_constituent].pct_change()
+
+    if drop_na:
+        price_data['nan_number']=price_data.isnull().sum(axis=1)
+        acceptable_nan_number =  price_data.shape[1]-bestofcount
+        price_data = price_data[price_data['nan_number']<=acceptable_nan_number]
+        price_data = price_data.drop(columns=['nan_number'])
+
+    if inception_date is None:
+        inception_date = min(price_data.index)
+    else :
+        inception_date = max(inception_date,min(price_data.index))
+
+
+
+    date_utility.add_rebalancing_datepart(price_data, 'Date', rebalancing_method= reshuffling_method, last=last)
+
+
+    print(f'backtesting with {reshuffling_method} method rebalancing since {inception_date} with early rebalancing {daily_rebalancing}')
+    print(f'date range before filtering {min(price_data.index)} {max(price_data.index)}')
+    price_data = price_data.loc[price_data.index >= inception_date, :]
+
+
+
+    print(f'date range after filtering by inception date {inception_date} and dropping all nas {min(price_data.index)} {max(price_data.index)}')
+
+    #assert inception_date in price_data.index
+    assert price_data.shape[0] > 0
+
+    np_price_data = price_data[constituents].values
+    np_previous_price_data =price_data[previous_prices_underlying].values
+    np_returns_data = price_data[returns_underlying].values
+
+    np_rebal_data = price_data['is_rebalancing'].values
+    return_assess_delay = 5
+    weights, indice_values = numba_indice_rebalancing_bestof_ok(daily_rebalancing, np_rebal_data, np_price_data,np_previous_price_data,np_returns_data, bestofcount = bestofcount, delay = return_assess_delay)
+
+    weights_df = pd.DataFrame(data = weights, index = price_data.index, columns = constituents)
+    weights_df = weights_df.fillna(0.)
+
+    date_utility.add_rebalancing_datepart(weights_df, 'Date', rebalancing_method = reshuffling_method, last=last)
+    weights_df['indice'] = indice_values
+
+    weights_df = weights_df.iloc[return_assess_delay:]
+
+    for me_constituent in constituents:
+        weights_df[me_constituent] = weights_df[me_constituent] / weights_df['indice']
+
+    weights_df['indice'] = weights_df['indice'] * initial_value
+
+    return weights_df
 
 def pre_numba_indice_rebalancing_bestof(price_data= None, inception_date=None, initial_value= 100., rebalancing_method = None , early_rebalancing = False, last = True, drop_na = True, bestofcount = 5):
 
@@ -327,6 +443,70 @@ def pre_numba_indice_rebalancing_bestof(price_data= None, inception_date=None, i
 
     return weights_df
 
+
+def pre_numba_indice_rebalancing_ok(price_data= None, inception_date=None, target_weights=None, initial_value= 100., reshuffling_method = None , daily_rebalancing = False, last = True, drop_all_na = True):
+    if target_weights is None:
+        target_weights = {}
+
+    constituents = list(target_weights.keys())
+    constituents.sort()
+
+    assert len([constituent for constituent in constituents if constituent in price_data.columns]) == len(constituents)
+    total_values = sum(target_weights.values())
+    assert total_values <= 1.001
+    cash_value = 1. - total_values
+    target_weights['cash'] = cash_value
+    price_data['cash'] = 1.
+
+    constituents = list(target_weights.keys())
+    constituents.sort()
+
+    previous_prices_underlying = []
+    for me_constituent in constituents:
+        previous_prices_constituent =  f'previous_price_{me_constituent}'
+        previous_prices_underlying.append(previous_prices_constituent)
+        price_data[previous_prices_constituent] = price_data[me_constituent].shift(1)
+
+    if inception_date is None:
+        inception_date = min(price_data.index)
+
+
+    date_utility.add_rebalancing_datepart(price_data, 'Date', rebalancing_method = reshuffling_method, last=last)
+
+
+    print(f'backtesting with {reshuffling_method} method rebalancing since {inception_date} with early rebalancing {daily_rebalancing}')
+    print(f'date range before filtering {min(price_data.index)} {max(price_data.index)}')
+    price_data = price_data.loc[price_data.index >= inception_date, :]
+
+    if drop_all_na:
+        price_data = price_data.dropna().copy()
+
+    print(f'date range after filtering by inception date {inception_date} and dropping all nas {min(price_data.index)} {max(price_data.index)}')
+
+    #assert inception_date in price_data.index
+    assert price_data.shape[0] > 0
+
+
+    np_price_data = price_data[constituents].values
+    np_previous_price_data =price_data[previous_prices_underlying].values
+    np_rebal_data = price_data['is_rebalancing'].values
+
+    target_weights = np.array([target_weights[consti] for consti in constituents])
+
+    weights, indice_values = numba_indice_rebalancing_ok(daily_rebalancing, np_rebal_data, np_price_data,np_previous_price_data, target_weights)
+
+    weights_df = pd.DataFrame(data = weights, index = price_data.index, columns = constituents)
+
+    date_utility.add_rebalancing_datepart(weights_df, 'Date', rebalancing_method = reshuffling_method, last=last)
+    weights_df['indice'] = indice_values
+    for me_constituent in constituents:
+        weights_df[me_constituent] = weights_df[me_constituent] / weights_df['indice']
+
+    weights_df['indice'] = weights_df['indice'] * initial_value
+
+    return weights_df
+
+
 def pre_numba_indice_rebalancing(price_data= None, inception_date=None, target_weights=None, initial_value= 100., rebalancing_method = None , early_rebalancing = False, last = True, drop_all_na = True):
     if target_weights is None:
         target_weights = {}
